<?php
/**
* A simple, standard CMS article page.
*
* @copyright 2013 by papaya Software GmbH - All rights reserved.
* @link http://www.papaya-cms.com/
* @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html GNU General Public License, version 2
*
* You can redistribute and/or modify this script under the terms of the GNU General Public
* License (GPL) version 2, provided that the copyright and license notes, including these
* lines, remain unmodified. papaya is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE.
*
* @package Papaya-Library
* @subpackage Modules-Standard
* @version $Id: SuggestionExamplePage.php 38835 2013-09-23 08:54:43Z weinert $
*/

/*
* A simple, standard CMS article page.
*
* @package Papaya-Library
* @subpackage Modules-Standard
*/
class PapayaModuleSuggestionExamplePage
  extends
    PapayaObject
  implements
    PapayaPluginAppendable,
    PapayaPluginQuoteable,
    PapayaPluginEditable,
    PapayaPluginCacheable {

  public $guid = '7bb5db40cdbeab1f847b14fe275c9e8f';

  private $_content = NULL;
  private $_cacheDefiniton = NULL;

  /**
   * Append the page output xml to the DOM.
   *
   * @see PapayaXmlAppendable::appendTo()
   */
  public function appendTo(PapayaXmlElement $parent) {
    $parent->appendElement('title', array(), $this->content()->get('title', ''));
    $parent->appendElement('teaser')->appendXml($this->content()->get('teaser', ''));
    $parent->appendElement('text')->appendXml($this->content()->get('text', ''));
  }

  /**
   * Append the teaser output xml to the DOM.
   *
   * @see PapayaXmlAppendable::appendTo()
   */
  public function appendQuoteTo(PapayaXmlElement $parent) {
    $parent->appendElement('title', array(), $this->content()->get('title', ''));
    $parent->appendElement('text')->appendXml($this->content()->get('teaser', ''));
  }

  /**
   * The content is an {@see ArrayObject} containing the stored data.
   *
   * @see PapayaPluginEditable::content()
   * @param PapayaPluginEditableContent $content
   * @return null|\PapayaPluginEditableContent
   */
  public function content(PapayaPluginEditableContent $content = NULL) {
    if (isset($content)) {
      $this->_content = $content;
    } elseif (NULL == $this->_content) {
      $this->_content = new PapayaPluginEditableContent();
      $this->_content->callbacks()->onCreateEditor = array($this, 'createEditor');
    }
    return $this->_content;
  }

  /**
   * The editor is used to change the stored data in the administration interface.
   *
   * In this case it the editor creates an dialog from a field definition.
   *
   * @see PapayaPluginEditableContent::editor()
   *
   * @param object $callbackContext
   * @param PapayaPluginEditableContent $content
   * @return PapayaPluginEditor
   */
  public function createEditor($callbackContext, PapayaPluginEditableContent $content) {
    $editor = new PapayaAdministrationPluginEditorFields(
      $content,
      array(
        'title' => array(
          'caption' => new PapayaUiStringTranslated('Title'),
          'mandatory' => TRUE,
          'type' => 'input',
          'parameters' => 400
        ),
        'teaser' => array(
          'caption' => new PapayaUiStringTranslated('Teaser'),
          'type' => 'input_suggest',
          'parameters' => 'modglyph.php?module='.urlencode($this->guid).'&src=script/suggest.xml'
        ),
        'test' => array(
          'caption' => new PapayaUiStringTranslated('test'),
          'type' => 'input_suggest',
          'parameters' => 'modglyph.php?module='.urlencode($this->guid).'&src=script/suggest2.xml'
        )
      )
    );
    $editor->papaya($this->papaya());
    return $editor;
  }

  /**
   * Define the code definition parameters for the output.
   *
   * @see PapayaPluginCacheable::cacheable()
   * @param PapayaCacheIdentifierDefinition $definition
   * @return PapayaCacheIdentifierDefinition
   */
  public function cacheable(PapayaCacheIdentifierDefinition $definition = NULL) {
    if (isset($definition)) {
      $this->_cacheDefiniton = $definition;
    } elseif (NULL == $this->_cacheDefiniton) {
      $this->_cacheDefiniton = new PapayaCacheIdentifierDefinitionPage();
    }
    return $this->_cacheDefiniton;
  }
}
